package cn.com.infosec.openssl;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.cms.ContentInfo;
import cn.com.infosec.asn1.pkcs.PrivateKeyInfo;
import cn.com.infosec.asn1.pkcs.RSAPrivateKeyStructure;
import cn.com.infosec.asn1.x509.DSAParameter;
import cn.com.infosec.jce.PKCS10CertificationRequest;
import cn.com.infosec.util.Strings;
import cn.com.infosec.util.encoders.Base64;
import cn.com.infosec.util.encoders.Hex;
import cn.com.infosec.x509.X509AttributeCertificate;
import cn.com.infosec.x509.X509V2AttributeCertificate;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.cert.CRLException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.interfaces.DSAParams;
import java.security.interfaces.DSAPrivateKey;
import java.security.interfaces.RSAPrivateCrtKey;
import java.security.interfaces.RSAPrivateKey;

public class PEMWriter extends BufferedWriter {
   private String provider;

   public PEMWriter(Writer out) {
      this(out, "INFOSEC");
   }

   public PEMWriter(Writer out, String provider) {
      super(out);
      this.provider = provider;
   }

   private void writeHexEncoded(byte[] bytes) {
      bytes = Hex.encode(bytes);

      for(int var2 = 0; var2 != bytes.length; ++var2) {
         this.write((char)bytes[var2]);
      }

   }

   private void writeEncoded(byte[] bytes) {
      byte bytes1 = 64;
      char[] var2 = new char[64];
      byte[] var3 = Base64.encode(bytes);

      for(int var4 = 0; var4 < var3.length; var4 += bytes1) {
         int var5;
         int var6;
         for(var5 = 0; var5 != bytes1 && (var6 = var4 + var5) < var3.length; ++var5) {
            var2[var5] = (char)var3[var6];
         }

         this.write(var2, 0, var5);
         this.newLine();
      }

   }

   private void writeHeader(String type) {
      this.write("-----BEGIN " + type + "-----");
      this.newLine();
   }

   private void writeFooter(String type) {
      this.write("-----END " + type + "-----");
      this.newLine();
   }

   public void writeObject(Object o) {
      byte[] o2;
      String var9;
      label53: {
         byte[] var2;
         String o1;
         byte[] var13;
         String var14;
         Object var10000;
         if (o instanceof X509Certificate) {
            var10000 = o;
            o1 = "CERTIFICATE";

            try {
               var13 = ((X509Certificate)var10000).getEncoded();
            } catch (CertificateEncodingException var6) {
               throw new IOException("Cannot encode object: " + var6.toString());
            }
         } else {
            if (!(o instanceof X509CRL)) {
               if (o instanceof KeyPair) {
                  this.writeObject(((KeyPair)o).getPrivate());
                  return;
               }

               if (o instanceof PrivateKey) {
                  PrivateKeyInfo var10;
                  var10 = new PrivateKeyInfo.<init>((ASN1Sequence)ASN1Object.fromByteArray(((Key)o).getEncoded()));
                  if (o instanceof RSAPrivateKey) {
                     o1 = "RSA PRIVATE KEY";
                     var2 = var10.getPrivateKey().getEncoded();
                     var14 = o1;
                     o2 = var2;
                     var9 = var14;
                  } else {
                     if (!(o instanceof DSAPrivateKey)) {
                        throw new IOException("Cannot identify private key");
                     }

                     PrivateKeyInfo var15 = var10;
                     var9 = "DSA PRIVATE KEY";
                     DSAParameter var3 = DSAParameter.getInstance(var15.getAlgorithmId().getParameters());
                     ASN1EncodableVector var4;
                     ASN1EncodableVector var16 = var4 = new ASN1EncodableVector;
                     var4.<init>();
                     var4.add(new DERInteger(0));
                     var4.add(new DERInteger(var3.getP()));
                     var4.add(new DERInteger(var3.getQ()));
                     var4.add(new DERInteger(var3.getG()));
                     BigInteger o3 = ((DSAPrivateKey)o).getX();
                     BigInteger var12 = var3.getG().modPow(o3, var3.getP());
                     var16.add(new DERInteger(var12));
                     var16.add(new DERInteger(o3));
                     o2 = (new DERSequence(var4)).getEncoded();
                  }
               } else if (o instanceof PublicKey) {
                  var9 = "PUBLIC KEY";
                  o2 = ((PublicKey)o).getEncoded();
               } else if (o instanceof X509AttributeCertificate) {
                  var9 = "ATTRIBUTE CERTIFICATE";
                  o2 = ((X509V2AttributeCertificate)o).getEncoded();
               } else if (o instanceof PKCS10CertificationRequest) {
                  var9 = "CERTIFICATE REQUEST";
                  o2 = ((PKCS10CertificationRequest)o).getEncoded();
               } else {
                  if (!(o instanceof ContentInfo)) {
                     throw new IOException("unknown object passed - can't encode.");
                  }

                  var9 = "PKCS7";
                  o2 = ((ContentInfo)o).getEncoded();
               }
               break label53;
            }

            var10000 = o;
            o1 = "X509 CRL";

            try {
               var13 = ((X509CRL)var10000).getEncoded();
            } catch (CRLException var5) {
               throw new IOException("Cannot encode object: " + var5.toString());
            }
         }

         var2 = var13;
         var14 = o1;
         o2 = var2;
         var9 = var14;
      }

      this.writeHeader(var9);
      this.writeEncoded(o2);
      this.writeFooter(var9);
   }

   public void writeObject(Object obj, String algorithm, char[] password, SecureRandom random) {
      if (obj instanceof KeyPair) {
         this.writeObject(((KeyPair)obj).getPrivate());
      } else {
         String var5 = null;
         byte[] var6 = null;
         BigInteger var7;
         BigInteger var18;
         if (obj instanceof RSAPrivateCrtKey) {
            var5 = "RSA PRIVATE KEY";
            RSAPrivateCrtKey var17 = (RSAPrivateCrtKey)obj;
            RSAPrivateCrtKey var10002 = var17;
            RSAPrivateCrtKey var10003 = var17;
            RSAPrivateCrtKey var10004 = var17;
            RSAPrivateCrtKey var10005 = var17;
            RSAPrivateCrtKey var10006 = var17;
            RSAPrivateCrtKey var10007 = var17;
            RSAPrivateCrtKey var10008 = var17;
            var18 = var17.getModulus();
            var7 = var10008.getPublicExponent();
            BigInteger var8 = var10007.getPrivateExponent();
            BigInteger var9 = var10006.getPrimeP();
            BigInteger var10 = var10005.getPrimeQ();
            BigInteger var11 = var10004.getPrimeExponentP();
            BigInteger var12 = var10003.getPrimeExponentQ();
            BigInteger var13 = var10002.getCrtCoefficient();
            var6 = (new RSAPrivateKeyStructure(var18, var7, var8, var9, var10, var11, var12, var13)).getEncoded();
         } else if (obj instanceof DSAPrivateKey) {
            var5 = "DSA PRIVATE KEY";
            DSAPrivateKey var19;
            DSAParams var20 = (var19 = (DSAPrivateKey)obj).getParams();
            ASN1EncodableVector var21;
            ASN1EncodableVector var10000 = var21 = new ASN1EncodableVector;
            var21.<init>();
            var21.add(new DERInteger(0));
            var21.add(new DERInteger(var20.getP()));
            var21.add(new DERInteger(var20.getQ()));
            var21.add(new DERInteger(var20.getG()));
            var18 = var19.getX();
            var7 = var20.getG().modPow(var18, var20.getP());
            var10000.add(new DERInteger(var7));
            var10000.add(new DERInteger(var18));
            var6 = (new DERSequence(var21)).getEncoded();
         }

         if (var5 != null && var6 != null) {
            String obj1;
            if ((obj1 = Strings.toUpperCase(algorithm)).equals("DESEDE")) {
               obj1 = "DES-EDE3-CBC";
            }

            byte[] algorithm1 = new byte[obj1.startsWith("AES-") ? 16 : 8];
            PEMWriter var22 = this;
            PEMWriter var23 = this;
            PEMWriter var24 = this;
            PEMWriter var25 = this;
            PEMWriter var26 = this;
            PEMWriter var27 = this;
            PEMWriter var28 = this;
            PEMWriter var10009 = this;
            PEMWriter var10010 = this;
            random.nextBytes(algorithm1);
            byte[] var14 = PEMUtilities.crypt(true, this.provider, var6, password, obj1, algorithm1);
            var10010.writeHeader(var5);
            var10009.write("Proc-Type: 4,ENCRYPTED");
            var28.newLine();
            var27.write("DEK-Info: " + obj1 + ",");
            var26.writeHexEncoded(algorithm1);
            var25.newLine();
            var24.newLine();
            var23.writeEncoded(var14);
            var22.writeFooter(var5);
         } else {
            throw new IllegalArgumentException("Object type not supported: " + obj.getClass().getName());
         }
      }
   }
}
